"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@auto-it/core");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const anymatch_1 = tslib_1.__importDefault(require("anymatch"));
const child_process_1 = require("child_process");
const rcFile = path_1.default.join(process.cwd(), '.all-contributorsrc');
const contributionTypes = [
    'blog',
    'bug',
    'business',
    'code',
    'content',
    'design',
    'doc',
    'eventOrganizing',
    'example',
    'financial',
    'fundingFinding',
    'ideas',
    'infra',
    'maintenance',
    'platform',
    'plugin',
    'projectManagement',
    'question',
    'review',
    'security',
    'talk',
    'test',
    'tool',
    'translation',
    'tutorial',
    'userTesting',
    'video;'
];
const defaultOptions = {
    exclude: [
        'dependabot-preview[bot]',
        'greenkeeper[bot]',
        'dependabot[bot]',
        'fossabot',
        'renovate',
        'renovate[bot]',
        'renovate-approve'
    ],
    types: {
        doc: ['**/*.mdx', '**/*.md', '**/docs/**/*', '**/documentation/**/*'],
        example: ['**/*.stories*', '**/*.story.*'],
        infra: ['**/.circle/**/*', '**/.github/**/*', '**/travis.yml'],
        test: ['**/*.test.*', '**/test/**', '**/__tests__/**'],
        code: ['**/src/**/*', '**/lib/**/*', '**/package.json', '**/tsconfig.json']
    }
};
/** Automatically add contributors as changelogs are produced. */
class AllContributorsPlugin {
    /** Initialize the plugin with it's options */
    constructor(options = {}) {
        /** The name of the plugin */
        this.name = 'All Contributors';
        this.options = {
            exclude: [...(defaultOptions.exclude || []), ...(options.exclude || [])],
            types: Object.assign(Object.assign({}, defaultOptions.types), options.types)
        };
    }
    /** Tap into auto plugin points. */
    apply(auto) {
        auto.hooks.afterAddToChangelog.tapPromise(this.name, async ({ commits }) => {
            const config = JSON.parse(fs_1.default.readFileSync(rcFile, 'utf8'));
            const authorContributions = {};
            let didUpdate = false;
            const commitsWithAllChangedFiles = await Promise.all(commits.map(async (commit) => {
                const extra = await core_1.execPromise('git', [
                    'show',
                    '--pretty=""',
                    '--name-only',
                    '--first-parent',
                    '-m',
                    commit.hash
                ]);
                commit.files = [
                    ...new Set([...commit.files, ...extra.split('\n')])
                ];
                return commit;
            }));
            // 1. Find all the authors and their contribution types
            commitsWithAllChangedFiles.forEach(commit => {
                const { authors } = commit;
                let { files } = commit;
                Object.keys(this.options.types || {})
                    .filter((type) => {
                    /** Determine if path is the contribution type */
                    const isType = (file) => anymatch_1.default(this.options.types[type] || [], file);
                    const isMatch = files.some(isType);
                    files = files.filter(file => !isType(file));
                    return isMatch;
                })
                    .forEach(contribution => {
                    authors.forEach(({ username }) => {
                        if (!username) {
                            return;
                        }
                        if (!authorContributions[username]) {
                            authorContributions[username] = new Set();
                        }
                        authorContributions[username].add(contribution);
                    });
                });
            });
            // 2. Determine if contributor has update
            Object.entries(authorContributions).forEach(([username, contributions]) => {
                const { contributions: old = [] } = config.contributors.find(contributor => contributor.login === username) || {};
                const hasNew = [...contributions].find(contribution => !old.includes(contribution));
                if (hasNew && !this.options.exclude.includes(username)) {
                    const newContributions = new Set([...old, ...contributions]);
                    didUpdate = true;
                    auto.logger.log.info(`Adding "${username}"'s contributions...`);
                    child_process_1.execSync(`npx all-contributors add ${username} ${[
                        ...newContributions
                    ].join(',')}`, { stdio: 'inherit' });
                }
            });
            if (didUpdate) {
                auto.logger.log.success('Updated contributors!');
            }
        });
    }
}
exports.default = AllContributorsPlugin;
//# sourceMappingURL=index.js.map